var extend = require('xtend');
var Promise = require('native-or-bluebird');
var path_1 = require('path');
var dependencies_1 = require('./lib/dependencies');
var compile_1 = require('./lib/compile');
var find_1 = require('./utils/find');
var fs_1 = require('./utils/fs');
var parse_1 = require('./utils/parse');
function install(options) {
    return dependencies_1.resolveTypeDependencies(options)
        .then(function (tree) {
        var cwd = path_1.dirname(tree.src);
        var queue = [];
        function addToQueue(deps, ambient) {
            for (var _i = 0, _a = Object.keys(deps); _i < _a.length; _i++) {
                var key = _a[_i];
                queue.push([key, deps[key], ambient]);
            }
        }
        addToQueue(tree.dependencies, false);
        addToQueue(tree.devDependencies, false);
        addToQueue(tree.ambientDependencies, true);
        return queue.reduce(function (result, _a) {
            var name = _a[0], tree = _a[1], ambient = _a[2];
            return result.then(function () { return installDependencyTree(tree, { cwd: cwd, name: name, ambient: ambient }); });
        }, Promise.resolve());
    });
}
exports.install = install;
function installDependency(dependency, options) {
    if (!options.name) {
        return Promise.reject(new Error('You must specify a name for the dependency'));
    }
    return find_1.findProject(options.cwd)
        .then(function (cwd) { return installTo(dependency, extend(options, { cwd: cwd })); }, function () { return installTo(dependency, options); });
}
exports.installDependency = installDependency;
function installTo(location, options) {
    var dependency = parse_1.parseDependency(location);
    return dependencies_1.resolveDependency(dependency, options)
        .then(function (tree) {
        if (tree.missing) {
            return Promise.reject(new TypeError('Unable to resolve dependency'));
        }
        return installDependencyTree(tree, {
            cwd: options.cwd,
            name: options.name,
            ambient: options.ambient || options.saveAmbient
        });
    })
        .then(function () { return writeToConfig(dependency, options); });
}
function installDependencyTree(tree, options) {
    return compile_1.default(tree, options)
        .then(function (definitions) { return fs_1.writeDependency(definitions, options); });
}
function writeToConfig(dependency, options) {
    if (!options.save && !options.saveDev && !options.saveAmbient) {
        return;
    }
    var location = parse_1.stringifyDependency(dependency);
    return fs_1.transformConfig(options.cwd, function (config) {
        if (options.save) {
            config.dependencies = extend(config.dependencies, (_a = {}, _a[options.name] = location, _a));
        }
        else if (options.saveDev) {
            config.devDependencies = extend(config.devDependencies, (_b = {}, _b[options.name] = location, _b));
        }
        else if (options.saveAmbient) {
            config.ambientDependencies = extend(config.ambientDependencies, (_c = {}, _c[options.name] = location, _c));
        }
        return config;
        var _a, _b, _c;
    });
}
//# sourceMappingURL=install.js.map